% ------------------------------------------------------------------------
%  Matlab code for computing Fourier-wavelet spectra (FWS)
%
%    [RW,RE,PERIOD,COI,SIGNIF,PW,PE] = ...
%           fourierwavelet(DATA,DT,DLON,ZWN,MIN_T,MAX_T,DJ)
%
%    Version 1.0 (released on 14 November 2022)
%    Version 1.1 (released on 22 March 2023)
%    Provided by Y. Yamazaki [https://orcid.org/0000-0002-7624-4752]
%    Available at URL: [https://igit.iap-kborn.de/yamazaki/fourierwavelet]
%
%    Reference: Yamazaki, Y. (2023): Matlab and Python software to  
%               compute Fourier-wavelet spectra (fourierwavelet v1.1)
%               using longitude-time data for studying global-scale
%               atmospheric waves
%
% *** Note ***
%
%    Created with Matlab ver. 9.11.0.1837725 (R2021b) Update 2
%
%    The following Matlab functions are required:
%
%      - wavelet.m
%      - wave_bases.m
%      - wave_signif.m
%      - chisquare_inv.m
%      - chisquare_solve.m
%
%    These Matlab functions are created by C. Torrence and G. Compo, and
%    are available at URL: [https://github.com/chris-torrence/wavelets].
%    See also [https://paos.colorado.edu/research/wavelets/] for more
%    details.
%
%    Reference: Torrence, C. and G. P. Compo, 1998: A Practical Guide to
%               Wavelet Analysis. Bull. Amer. Meteor. Soc., 79, 61-78.
%
% *** **** ***
% ------------------------------------------------------------------------
% INPUTS:
%
%    DATA is a space-time data matrix [time x longitude]. Each column
%    corresponds to a time series at a fixed longitude. Each row
%    corresponds to a longitude distribution at a fixed time. Make sure
%    that size(DATA,1) gives the length in time, and size(DATA,2) gives
%    the length in longitude. The data need to be equally distributed in
%    time and longitude. Gaps should be filled by NaN. Also, there should
%    be no repetition in longitude. For example, if the first and last
%    columns are assigned to the same longitude (e.g., 0 and 360 deg),
%    remove the last column, i.e., DATA = DATA(:,end-1), to avoid the
%    repetition.
%
%    DT is sampling time (arbitrary units).
%
%    DLON is longitude spacieng (in units of degrees).
%
%    ZWN is the zonal wavenumber (an integer value, >=0).
%
% ------------------------------------------------------------------------
% OUTPUTS:
%
%    RW is a matrix [PERIOD x time] of the amplitude of the
%    westward-propagating wave component with zonal wavenuber ZWN.
%
%    RE is the same as RW except for the eastward-propagating wave
%    component.
%
%    PERIOD is a vector of Fourier periods (in time units).
%
%    COI is a vector for the Cone-of-Influence.
%    It gives a vector that contains the maximum period of useful
%    information at that particular time.
%    Periods greater than COI are subject to edge effects.
%    This can be used to plot COI lines on a contour plot, for example:
%
%           >> contour(time,log2(period),Rw), hold on
%           >> plot(time,log2(coi),'k')
%
%    SIGNIF is a matrix [PERIOD x time] in the same size as RW and RE. 
%    RW (or RE) is above the 95% confidence for a red noise background
%    (lag-1 autocorrelation = 0.72) where RW (or RE) > SIGNIF.
%    This can be used to plot the 95% confidence lines, for example:
%
%           >> contour(time,log2(period),Rw./signif,1,'m') 
%
%    PW is a matrix [PERIOD x time] of the phase of the
%    westward-propagating wave component with zonal wavenumber ZWN.
%
%    PE is the same as RW except for the eastward-propagating wave
%    component.
%
% ------------------------------------------------------------------------
% OPTIONAL INPUTS:
%
% *** Note *** 
%
%     Setting any of the following to -1 will cause the default value to
%     be used.
%
% *** **** ***
%
%    MIN_T and MAX_T are the minimum and minimum PERIOD to be included, in 
%    the same time units as DT. If not specified, the results are output
%    for full PERIOD range.
%
%    DJ is the spacing between discrete scales. Default is 0.1.
%    A smaller value will give better scale resolution, but the
%    computation takes more time.
%
% ------------------------------------------------------------------------
% LICENSE:
%
%    Copyright (C) 2022, Yosuke Yamazaki
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
% 
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
%    GNU General Public License for more details.
% 
%    You should have received a copy of the GNU General Public License
%    along with this program. If not, see <https://www.gnu.org/licenses/>.
%
% *** Contact *** 
%
%     Dr. Yosuke Yamazaki
%     Leibniz Institute of Atmospheric Physics (IAP) at the University of
%     Rostock, Schlossstraße 6, 18225 Kühlungsborn, Germany
%     E-mail: yamazaki[AT]iap-kborn[DOT]de
%
% *** ******* ***
%
%-------------------------------------------------------------------------
function [Rw,Re,period,coi,signif,Pw,Pe] = ...
             fourierwavelet(data,dt,dlon,zwn,min_t,max_t,dj)
% ------------------------------------------------------------------------
% check inputs
if (nargin < 7), dj  = 0.1; end
if (nargin < 6), max_t = inf; end
if (nargin < 5), min_t = -1; end
if nargin < 4
    em0 = 'Must inpu the following:';
    em1 = '(1) DATA, data matrix [time x longitude]';
    em2 = '(2) DT, sampling time (arbitrary unit)';
    em3 = '(3) DLON, longitude spacing (deg.)';
    em4 = '(4) ZWN, zonal wavenumber';
    error('%s\n %s\n %s\n %s \n %s',em0,em1,em2,em3,em4)
end
if min_t>max_t
    error('MAX_T must be larger than MIN_T.')
end
if (dj==-1), dj = 0.1; end
if (max_t==-1), max_t = inf; end
% ------------------------------------------------------------------------
% check redundancy in longitude
if min(data(:,1)==data(:,end))
    fprintf(2,'The first and last columns of the data are identical.\n')
    fprintf(2,['If they are assigned to the same longitude'...
        '(e.g., 0 and 360 deg),\n'])
    fprintf(2,'remove the last column, i.e., DATA = DATA(:,end-1).\n')
end
% ------------------------------------------------------------------------
% check the availability of the wavelet package
wp1 = exist('wavelet.m','file');
wp2 = exist('wave_bases.m','file');
wp3 = exist('wave_signif.m','file');
wp4 = exist('chisquare_inv.m','file');
if wp1==0 || wp2==0 || wp3==0 || wp4==0
    em0 = 'The following Matlab functions are required:';
    em1 = '(1) wavelet.m';
    em2 = '(2) wave_bases.m';
    em3 = '(3) wave_signif.m';
    em4 = '(4) chisquare_inv.m';
    em5 = '(5) chisquare_solve.m';
    em6 = 'These Matlab functions are available at URL:';
    em7 = 'https://github.com/chris-torrence/wavelets';
    error('%s\n %s\n %s\n %s\n %s\n %s\n %s\n %s',...
        em0,em1,em2,em3,em4,em5,em6,em7)
end
% ------------------------------------------------------------------------
% reconstruction of time and longitude
n = size(data,1); % data length in time
L = size(data,2); % data length in longitude
chk_nan = max(max(isnan(data),[],2)); % 1 if NaN is detected
% ------------------------------------------------------------------------
% Fourier analysis in longitude
if chk_nan==1
    % least-squares method (only when NaN exists in data)
    Ck = mean(data,2,'omitnan');
    Ck = Ck - mean(Ck,'omitnan');
    Sk = zeros(size(Ck));
    if zwn == 0
        Ck(isnan(Ck))=0;
    else
        lon = (0:dlon:size(data,2)*dlon-dlon)'; % reconstructed longitude
        M = [cos(2*pi*(abs(zwn))*lon/360),sin(2*pi*(abs(zwn))*lon/360)];
        for l = 1:n
            t_data = data(l,:)';
            co = M(~isnan(t_data),:)\t_data(~isnan(t_data));
            Ck(l) = co(1);
            Sk(l) = co(2);
        end
    end
else
    % fft method
    Ck = mean(data,2);
    Ck = Ck - mean(Ck);
    Sk = zeros(size(Ck));
    if zwn ~= 0
        for l = 1:n
            f_data = fft(data(l,:)')/L;
            Ck(l) =  2*real(f_data(zwn+1));
            Sk(l) = -2*imag(f_data(zwn+1));
        end
    end
end
% ------------------------------------------------------------------------
% wavelet analysis in time
param = 6;
[Ck_w0,period,scale,coi] = wavelet(Ck,dt,1,dj,2*dt,...
    fix((log(n*dt/(2*dt))/log(2))/dj),'MORLET',param);
Sk_w0 = wavelet(Sk,dt,1,dj,2*dt,...
    fix((log(n*dt/(2*dt))/log(2))/dj),'MORLET',param);
Ck_w = Ck_w0.*(sqrt(dt./scale)'*ones(1,n));
Sk_w = Sk_w0.*(sqrt(dt./scale)'*ones(1,n));
% ------------------------------------------------------------------------
% calculation of amplitude and phase
Ak =  real(Ck_w);
Bk = -imag(Ck_w);
ak =  real(Sk_w);
bk = -imag(Sk_w);
Rw = .5*sqrt((Ak-bk).^2 + ( Bk+ak).^2); % amplitude, westward
Re = .5*sqrt((Ak+bk).^2 + ( Bk-ak).^2); % amplitude, eastward
Pw =  atan2( Bk+ak,Ak-bk); % phase, westward
Pe =  atan2( Bk-ak,Ak+bk); % phase, eastward
n_min = find(period<min_t, 1, 'last' );
n_max = find(period>max_t, 1, 'first');
if isempty(n_min), n_min = 1; end
if isempty(n_max), n_max = length(period); end
period = period(n_min:n_max);
Rw = Rw(n_min:n_max,:);
Re = Re(n_min:n_max,:);
Pw = Pw(n_min:n_max,:);
Pe = Pe(n_min:n_max,:);
% ------------------------------------------------------------------------
% 0.05 significance levels
signif = wave_signif(max([var(Ck),var(Sk)]),dt,...
    scale(n_min:n_max),0,0.72,-1,-1,'MORLET');
% signif = signif;
signif = .5*sqrt(signif.*sqrt(dt./scale(n_min:n_max)));
signif = signif'*ones(1,n);
% ------------------------------------------------------------------------
end