% ------------------------------------------------------------------------
%
%  Matlab example script for computing Fourier-wavelet sepctra using
%  geopotential height at 50N and 0.01 hPa during January-May 2016 as
%  obtained from the SD/WACCM-X model.
%
%  This software is provided by Y. Yamazaki and is available at URL:
%  [https://igit.iap-kborn.de/yamazaki/fourierwavelet].
%
%    Reference: Yamazaki, Y. (2023): Matlab and Python software to  
%               compute Fourier-wavelet spectra (fourierwavelet v1.1)
%               using longitude-time data for studying global-scale
%               atmospheric waves
%
%  The SD/WACCM-X simulation is that presented by F. Gasperini et al. and
%  the data are available at URL: [https://doi.org/10.26024/5b58-nc53].
%
%  Reference: Gasperini, F., Liu, H., & McInerney, J. (2020). Preliminary
%             evidence of Madden-Julian Oscillation effects on ultrafast
%             tropical waves in the thermosphere. Journal of Geophysical
%             Research: Space Physics, 125, e2019JA027649.
%             https://doi.org/10.1029/2019JA027649.
%
% ------------------------------------------------------------------------

% ------------------------------------------------------------------------
% reading input data

GPH = load('SD-WACCM-X_v2.1_GPH.dat');
dt = 1;     % time grid resoltuion (day)
dlon = 2.5; % longitude grid resolution (deg)
zwn = 1;    % zonal wavenumber

% ------------------------------------------------------------------------
% creating vectors for time and longitude

t = 0:dt:size(GPH,1)-1; % time
lon = 0:dlon:360-dlon;  % longitude

% ------------------------------------------------------------------------
% performing Fourier-wavelet analysis

[Rw,Re,period,coi,signif] = fourierwavelet(GPH,dt,dlon,zwn);

% -------------------
% using all optional inputs & outputs
%
% min_t = 3;
% max_t = 30;
% dj    = 0.05;
%
% [Rw,Re,period,coi,signif,Pw,Pe] = ...
%     fourierwavelet(GPH,dt,dlon,zwn,min_t,max_t,dj);
%

% ------------------------------------------------------------------------
% plotting

figure

% -------------------
% plotting input data

ax(1) = subplot(3,1,1);
pcolor(t,lon,GPH')
xlabel('Time (days from 1 Jan 2016)')
ylabel('Longitude (deg.)')
title('SD/WACCM-X geopotential height (km) at 50N and 0.01 hPa')
colorbar
colormap(ax(1),jet)
shading flat
set(gca,'layer','top','xtick',30:30:150,'tickdir','out')

% -------------------
% plotting Fourier-wavelet spectrum for the W1 component

ax(2) = subplot(3,1,2);
contourf(t,period,Rw,0:.1:.7,'linecolor','none')
hold on
contour(t,period,Rw./signif,[1,1],'w','linewidth',2)
plot(t,coi,'w--','linewidth',2)
set(gca,'ylim',[4,28])
xlabel('Time (days from 1 Jan 2016)')
ylabel('Period (days)')
title('Fourier-wavelet sepctrum for W1 (km)')
clim([0,.7])
colorbar
colormap(ax(2),parula)
set(gca,'layer','top','xtick',30:30:150,'tickdir','out')

% -------------------
% plotting Fourier-wavelet spectrum for the E1 component

ax(3) = subplot(3,1,3);
contourf(t,period,Re,0:.1:.7,'linecolor','none')
hold on
contour(t,period,Re./signif,[1,1],'w','linewidth',2)
plot(t,coi,'w--','linewidth',2)
set(gca,'ylim',[4,28])
xlabel('Time (days from 1 Jan 2016)')
ylabel('Period (days)')
title('Fourier-wavelet sepctrum for E1 (km)')
clim([0,.7])
colorbar
colormap(ax(3),parula)
set(gca,'layer','top','xtick',30:30:150,'tickdir','out')

% ------------------------------------------------------------------------
