% Matlab code for recreating Figure 1 of Yamazaki (2023)
% 
%  Reference:
%   Yamazaki, Y. (2023): A method to derive Fourier-wavelet spectra for
%   the characterization of global-scale waves in the mesosphere and lower
%   thermosphere, and its Matlab and Python software (fourierwavelet v1.1).
%   submitted to Geoscientific Model Development (GMD).
%   MS No.: egusphere-2022-1275.
%
%  Figure caption:
%    (a) Synthetic data containing wave_A (westward-propagating with zonal
%        wavenumber 2, W2) and wave_B (eastward-propagating with zonal
%        wavenumber 3, E3), along with noise.
%    (b) Amplitude of wave_A.
%    (c) Phase of wave_A (magenta line), and Fourier-wavelet amplitude
%        spectrum for W2 (contour plot). The white curves indicate the
%        95% confidence level, while the white dashed lines show the cone
%        of influence.
%    (d) Same as (c) except that the amplitude spectrum is derived with
%        the least-squares fitting method.
%    (e–g) Same as (b–d) but for wave_B. The amplitude spectra are for E3.
%
% *** Note ***
%
%    The following Matlab functions are required:
%
%      - fourierwavelet.m
%      - wavelet.m
%      - wave_bases.m
%      - wave_signif.m
%      - chisquare_inv.m
%      - chisquare_solve.m
%
% *** **** ***
% ------------------------------------------------------------------------

load('recreation_fig1_data.mat')

t = 1:365;
lon = (-180:2.5:180-2.5);

dt = mean(diff(t));
dlon = mean(diff(lon));

% ------------------------------------------------------------------------

% Fourier-wavelet method
zwn = 2;
[Rw2,~,period2,coi2,signif2] = ...
             fourierwavelet(data,dt,dlon,zwn,4,30,0.05);
zwn = 3;
[~,Re3,period3,coi3,signif3] = ...
             fourierwavelet(data,dt,dlon,zwn,4,30,0.05);

% ------------------------------------------------------------------------

% Least-squares fitting method
Rw2_lsf = NaN(length(period2),length(t));
Re3_lsf = NaN(length(period3),length(t));
[tg,lg] = meshgrid(t,lon);
for t0 = 1:length(period2) % wave period
    T = period2(t0);
    wl = T*3; % window width
    j = 1;
    while length(-j:j)<wl
        j = j + 1;
    end
    for o = 1:length(t)
        if t(o)-j>0 && t(o)+j<=365
            data2 = data(t(o)-j:t(o)+j,:)';
            tg2 = tg(:,t(o)-j:t(o)+j);
            lg2 = lg(:,t(o)-j:t(o)+j);
            data3 = reshape(data2,size(data2,1)*size(data2,2),1);
            tg3   = reshape(tg2,size(tg2,1)*size(tg2,2),1);
            lg3   = reshape(lg2,size(lg2,1)*size(lg2,2),1);
            M_w2 = [cos((2*pi*tg3/T)+( 2*2*pi*lg3/360)),...
                    sin((2*pi*tg3/T)+( 2*2*pi*lg3/360))];
            M_e3 = [cos((2*pi*tg3/T)+(-3*2*pi*lg3/360)),...
                    sin((2*pi*tg3/T)+(-3*2*pi*lg3/360))];
            co_w2 = M_w2\data3;
            co_e3 = M_e3\data3;
            Rw2_lsf(t0,o) = sqrt(co_w2(1)^2+co_w2(2)^2);
            Re3_lsf(t0,o) = sqrt(co_e3(1)^2+co_e3(2)^2);
        end
    end
end

% ------------------------------------------------------------------------

% figure 1a
fig1a = figure;
pcolor(t,lon,data')
shading flat
title('Figure 1a')

% figure 1b-1d
fig1b = figure;
subplot(3,1,1)
plot(t,A2,'k')
set(gca,'xlim',[1,365])
title('Figure 1b')
subplot(3,1,2)
pcolor(t,period2,Rw2)
shading flat
hold on
contour(t,period2,Rw2./signif2,[1,1],'w')
plot(t,coi2,'w--')
plot(t,T2)
set(gca,'xlim',[1,365])
caxis([0,1])
title('Figure 1c')
subplot(3,1,3)
pcolor(t,period2,Rw2_lsf)
shading flat
hold on
plot(t,coi2,'w--')
plot(t,T2)
set(gca,'xlim',[1,365])
caxis([0,1])
title('Figure 1d')

% figure 1e-1g
fig1c = figure;
subplot(3,1,1)
plot(t,A3,'k')
set(gca,'xlim',[1,365])
title('Figure 1e')
subplot(3,1,2)
pcolor(t,period3,Re3)
shading flat
hold on
contour(t,period3,Re3./signif3,[1,1],'w')
plot(t,coi2,'w--')
plot(t,T3)
set(gca,'xlim',[1,365])
caxis([0,1])
title('Figure 1f')
subplot(3,1,3)
pcolor(t,period3,Re3_lsf)
shading flat
hold on
plot(t,coi2,'w--')
plot(t,T3)
set(gca,'xlim',[1,365])
caxis([0,1])
title('Figure 1g')
