% Matlab code for recreating Figure 2 of Yamazaki (2023)
% 
%  Reference:
%   Yamazaki, Y. (2023): A method to derive Fourier-wavelet spectra for
%   the characterization of global-scale waves in the mesosphere and lower
%   thermosphere, and its Matlab and Python software (fourierwavelet v1.1).
%   submitted to Geoscientific Model Development (GMD).
%   MS No.: egusphere-2022-1275.
%
%  Figure caption:
%   GAIA model simulation for the period August–October 2019.
%   (a) Polar temperature and zonal mean zonal wind velocity
%       at 60-deg N at 10 hPa.
%   (b) Zonal wind velocity over the equator at a height of 100 km.
%   (c–h) Fourier-wavelet spectra of the equatorial zonal wind velocity
%         at 100 km for
%   (c) the westward-propagating zonal wavenumber 1 (W1) component,
%   (d) the eastward-propagating zonal wavenumber 1 (E1) component,
%   (e) the westward-propagating zonal wavenumber 2 (W2) component,
%   (f) the eastward-propagating zonal wavenumber 2 (E2) component,
%   (g) the westward-propagating zonal wavenumber 3 (W3) component and
%   (h) the eastward-propagating zonal wavenumber 3 (E3) component.
%   The white curves indicate the 95% confidence level, while the white
%   dashed lines show the cone of influence.
%
%  Data reference:
%   Yamazaki, Y. & Miyoshi, Y. (2020): Simulation data from GAIA (Ground-
%   to-topside model of Atmosphere and Ionosphere for Aeronomy) for the
%   September 2019 sudden stratospheric warming event. GFZ Data Services.
%   https://doi.org/10.5880/GFZ.2.3.2020.004
% 
%  Simulation reference:
%   Miyoshi, Y., & Yamazaki, Y. (2020). Excitation mechanism of
%   ionospheric 6‐day oscillation during the 2019 September sudden
%   stratospheric warming event. Journal of Geophysical Research: Space
%   Physics, 125(9). https://doi.org/10.1029/2020JA028283
%
% *** Note ***
%
%    The following Matlab functions are required:
%
%      - fourierwavelet.m
%      - wavelet.m
%      - wave_bases.m
%      - wave_signif.m
%      - chisquare_inv.m
%      - chisquare_solve.m
%
% *** **** ***
% ------------------------------------------------------------------------

load('recreation_fig2_data.mat')

dt = mean(diff(t));
dlon = mean(diff(lon));

min_t = .3;
max_t = 30;

dj = 0.2;

zwn = 1;
[Rw1,Re1,period,coi,signif1] = ...
    fourierwavelet(data,dt,dlon,zwn,min_t,max_t,dj);

zwn = 2;
[Rw2,Re2,~,~,signif2] = ...
    fourierwavelet(data,dt,dlon,zwn,min_t,max_t,dj);

zwn = 3;
[Rw3,Re3,~,~,signif3] = ...
    fourierwavelet(data,dt,dlon,zwn,min_t,max_t,dj);

% ------------------------------------------------------------------------

% figure 2a
fig2a = figure;
plot(t0,Tp,'k')
hold on
plot(t0,U60,'r')
set(gca,'xlim',[0,92])
title('Figure 2a')

% figure 2b
fig2b = figure;
pcolor(t,lon,data')
shading flat
caxis([-200,200])
title('Figure 2b')

% figure 2c
fig2c = figure;
pcolor(t,period,Rw1)
shading flat
hold on
contour(t,period,Rw1./signif1,[1,1],'w')
plot(t,coi,'w--')
caxis([0,20])
title('Figure 2c')

% figure 2d
fig2d = figure;
pcolor(t,period,Re1)
shading flat
hold on
contour(t,period,Re1./signif1,[1,1],'w')
plot(t,coi,'w--')
caxis([0,20])
title('Figure 2d')

% figure 2e
fig2e = figure;
pcolor(t,period,Rw2)
shading flat
hold on
contour(t,period,Rw2./signif2,[1,1],'w')
plot(t,coi,'w--')
caxis([0,20])
title('Figure 2e')

% figure 2f
fig2f = figure;
pcolor(t,period,Re2)
shading flat
hold on
contour(t,period,Re2./signif2,[1,1],'w')
plot(t,coi,'w--')
caxis([0,20])
title('Figure 2f')

% figure 2g
fig2g = figure;
pcolor(t,period,Rw3)
shading flat
hold on
contour(t,period,Rw3./signif3,[1,1],'w')
plot(t,coi,'w--')
caxis([0,20])
title('Figure 2g')

% figure 2h
fig2h = figure;
pcolor(t,period,Re3)
shading flat
hold on
contour(t,period,Re3./signif3,[1,1],'w')
plot(t,coi,'w--')
caxis([0,20])
title('Figure 2h')
