% Matlab code for recreating Figure 3 of Yamazaki (2023)
% 
%  Reference:
%   Yamazaki, Y. (2023): A method to derive Fourier-wavelet spectra for
%   the characterization of global-scale waves in the mesosphere and lower
%   thermosphere, and its Matlab and Python software (fourierwavelet v1.1).
%   submitted to Geoscientific Model Development (GMD).
%   MS No.: egusphere-2022-1275.
%
%  Figure caption:
%   SD/WACCM-X model simulation for the period January–February 2009.
%   (a) Polar temperature and zonal mean zonal wind velocity at 60-deg N
%       at 10 hPa.
%   (b) Fourier-wavelet spectrum of the zonal wind velocity at 50-deg N
%       and 110 km. The white curves indicate the 95% confidence level,
%       while the white dashed lines show the cone of influence.
%   (c) Amplitude of the migrating semidiurnal tide in the zonal wind
%       velocity at 110 km as determined by the Fourier-wavelet technique.
%   (d) Amplitude of the migrating diurnal tide in the zonal wind velocity
%       at 95 km as determined by the Fourier-wavelet technique.
%
%  Data reference:
%   Siddiqui, T. (2020) WACCM-X simulations - 2009 SSW, Mendeley Data,
%   V1, doi: 10.17632/47pnw8pgmk.1
%   https://data.mendeley.com/datasets/47pnw8pgmk/1
%
%  Simulation reference
%   Siddiqui, T. A., Yamazaki, Y., Stolle, C., Maute, A., Laštovička, J.,
%   Edemskiy, I. K., Mošna, Z. & Sivakandan, M. (2021). Understanding
%   the total electron content variability over Europe during 2009 and
%   2019 SSWs. Journal of Geophysical Research: Space Physics, 126(9),
%   e2020JA028751. https://doi.org/10.1029/2020JA028751
%
% *** Note ***
%
%    The following Matlab functions are required:
%
%      - fourierwavelet.m
%      - wavelet.m
%      - wave_bases.m
%      - wave_signif.m
%      - chisquare_inv.m
%      - chisquare_solve.m
%
% *** **** ***
% ------------------------------------------------------------------------

load('recreation_fig3_data.mat')

[Rw2,~,period,coi,signif] = ...
         fourierwavelet(data,1,2.5,2,4,30,0.05);

% ------------------------------------------------------------------------

% figure 3a
fig3a = figure;
plot(t0,Tp,'k')
hold on
plot(t0,U60,'r')
set(gca,'xlim',[0,59])
title('Figure 3a')

% figure 3b
fig3b = figure;
pcolor(t,period,Rw2)
shading flat
hold on
contour(t,period,Rw2./signif,[1,1],'w')
plot(t,coi,'w--')
caxis([0,70])
set(gca,'xlim',[0,59])
title('Figure 3b')

% figure 3c
fig3c = figure;
pcolor(t,lat,SW2)
shading flat
hold on
contour(t,lat,SW2./SW2s,[1,1],'w')
caxis([0,70])
set(gca,'xlim',[0,59])
title('Figure 3c')

% figure 3d
fig3d = figure;
pcolor(t,lat,DW1)
shading flat
hold on
contour(t,lat,DW1./DW1s,[1,1],'w')
caxis([0,70])
set(gca,'xlim',[0,59])
title('Figure 3d')
