% Matlab code for recreating Figure 4 of Yamazaki (2023)
% 
%  Reference:
%   Yamazaki, Y. (2023): A method to derive Fourier-wavelet spectra for
%   the characterization of global-scale waves in the mesosphere and lower
%   thermosphere, and its Matlab and Python software (fourierwavelet v1.1).
%   submitted to Geoscientific Model Development (GMD).
%   MS No.: egusphere-2022-1275.
%
%  Figure caption:
%   SD/WACCM-X model simulation for the period January–May 2016.
%   (a) Polar temperature and zonal mean zonal wind velocity at 60-deg N
%       at 10 hPa.
%   (b) Geopotential height at 55-deg N at 0.01 hPa.
%   (c–d) Fourier-wavelet spectra of the geopotential height at 55-deg N
%         at 0.01 hPa for
%   (c) the westward-propagating zonal wavenumber 1 (W1) component and
%   (d) the eastward-propagating zonal wavenum- ber 1 (E1) component.
%   (e–f) Height profiles of (e) amplitude and (f) phase of the W1
%         component at a period of 10 days at 55-deg N and 0.01 hPa
%         on Day 72 as determined by the Fourier-wavelet technique.
%
%  Data reference:
%   SD WACCM-X v2.1. https://doi.org/10.26024/5b58-nc53
% 
%  Simulation reference:
%   Gasperini, F., Liu, H., & McInerney, J. (2020). Preliminary evidence
%   of Madden‐Julian Oscillation effects on ultrafast tropical waves in
%   the thermosphere. Journal of Geophysical Research: Space Physics,
%   125(5), e2019JA027649. https://doi.org/10.1029/2019JA027649
%
% *** Note ***
%
%    The following Matlab functions are required:
%
%      - fourierwavelet.m
%      - wavelet.m
%      - wave_bases.m
%      - wave_signif.m
%      - chisquare_inv.m
%      - chisquare_solve.m
%
% *** **** ***
% ------------------------------------------------------------------------

load('recreation_fig4_data.mat')

[Rw1,Re1,period,coi,signif1] = ...
     fourierwavelet(data,1,2.5,1,4,28);

% ------------------------------------------------------------------------

% figure 4a
fig4a = figure;
plot(t,Tp,'k')
hold on
plot(t,U60,'r')
set(gca,'xlim',[0,151])
title('Figure 4a')

% figure 4b
fig4b = figure;
pcolor(t,lon,data')
shading flat
caxis([74,80])
set(gca,'xlim',[0,151])
title('Figure 4b')

% figure 4c
fig4c = figure;
pcolor(t,period,Rw1)
shading flat
hold on
contour(t,period,Rw1./signif1,[1,1],'w')
plot(t,coi,'w--')
caxis([0,.7])
set(gca,'xlim',[0,151])
title('Figure 4c')

% figure 4d
fig4d = figure;
pcolor(t,period,Re1)
shading flat
hold on
contour(t,period,Re1./signif1,[1,1],'w')
plot(t,coi,'w--')
caxis([0,.7])
set(gca,'xlim',[0,151])
title('Figure 4d')

% figures 4e & 4f
fig4ef = figure;
subplot(1,2,1)
plot(amp,7*log(1000./lev),'k')
set(gca,'ylim',[15,95])
title('Figure 4e')
subplot(1,2,2)
plot(phs,7*log(1000./lev),'k')
set(gca,'ylim',[15,95])
title('Figure 4f')
